@@ -1,11 +1,11 @@
-import json
 from functools import wraps
 
 from django.conf import settings
 
-from auditlog.context import threadlocal
+from auditlog.context import auditlog_disabled
 from auditlog.diff import model_instance_diff
 from auditlog.models import LogEntry
+from auditlog.signals import post_log, pre_log
 
 
 def check_disable(signal_handler):
@@ -17,7 +17,11 @@ def check_disable(signal_handler):
 
     @wraps(signal_handler)
     def wrapper(*args, **kwargs):
-        if not getattr(threadlocal, "auditlog_disabled", False) and not (
+        try:
+            auditlog_disabled_value = auditlog_disabled.get()
+        except LookupError:
+            auditlog_disabled_value = False
+        if not auditlog_disabled_value and not (
             kwargs.get("raw") and settings.AUDITLOG_DISABLE_ON_RAW_SAVE
         ):
             signal_handler(*args, **kwargs)
@@ -33,12 +37,12 @@ def log_create(sender, instance, created, **kwargs):
     Direct use is discouraged, connect your model through :py:func:`auditlog.registry.register` instead.
     """
     if created:
-        changes = model_instance_diff(None, instance)
-
-        LogEntry.objects.log_create(
-            instance,
+        _create_log_entry(
             action=LogEntry.Action.CREATE,
-            changes=json.dumps(changes),
+            instance=instance,
+            sender=sender,
+            diff_old=None,
+            diff_new=instance,
         )
 
 
@@ -49,23 +53,17 @@ def log_update(sender, instance, **kwargs):
 
     Direct use is discouraged, connect your model through :py:func:`auditlog.registry.register` instead.
     """
-    if instance.pk is not None:
-        try:
-            old = sender.objects.get(pk=instance.pk)
-        except sender.DoesNotExist:
-            pass
-        else:
-            new = instance
-            update_fields = kwargs.get("update_fields", None)
-            changes = model_instance_diff(old, new, fields_to_check=update_fields)
-
-            # Log an entry only if there are changes
-            if changes:
-                LogEntry.objects.log_create(
-                    instance,
-                    action=LogEntry.Action.UPDATE,
-                    changes=json.dumps(changes),
-                )
+    if not instance._state.adding and instance.pk is not None:
+        update_fields = kwargs.get("update_fields", None)
+        old = sender._default_manager.filter(pk=instance.pk).first()
+        _create_log_entry(
+            action=LogEntry.Action.UPDATE,
+            instance=instance,
+            sender=sender,
+            diff_old=old,
+            diff_new=instance,
+            fields_to_check=update_fields,
+        )
 
 
 @check_disable
@@ -76,12 +74,12 @@ def log_delete(sender, instance, **kwargs):
     Direct use is discouraged, connect your model through :py:func:`auditlog.registry.register` instead.
     """
     if instance.pk is not None:
-        changes = model_instance_diff(instance, None)
-
-        LogEntry.objects.log_create(
-            instance,
+        _create_log_entry(
             action=LogEntry.Action.DELETE,
-            changes=json.dumps(changes),
+            instance=instance,
+            sender=sender,
+            diff_old=instance,
+            diff_new=None,
         )
 
 
@@ -92,14 +90,62 @@ def log_access(sender, instance, **kwargs):
     Direct use is discouraged, connect your model through :py:func:`auditlog.registry.register` instead.
     """
     if instance.pk is not None:
-
-        LogEntry.objects.log_create(
-            instance,
+        _create_log_entry(
             action=LogEntry.Action.ACCESS,
-            changes="null",
+            instance=instance,
+            sender=sender,
+            diff_old=None,
+            diff_new=None,
+            force_log=True,
         )
 
 
+def _create_log_entry(
+    action, instance, sender, diff_old, diff_new, fields_to_check=None, force_log=False
+):
+    pre_log_results = pre_log.send(
+        sender,
+        instance=instance,
+        action=action,
+    )
+
+    if any(item[1] is False for item in pre_log_results):
+        return
+
+    error = None
+    log_entry = None
+    changes = None
+    try:
+        changes = model_instance_diff(
+            diff_old, diff_new, fields_to_check=fields_to_check
+        )
+
+        if force_log or changes:
+            log_entry = LogEntry.objects.log_create(
+                instance,
+                action=action,
+                changes=changes,
+                force_log=force_log,
+            )
+    except BaseException as e:
+        error = e
+    finally:
+        if log_entry or error:
+            post_log.send(
+                sender,
+                instance=instance,
+                instance_old=diff_old,
+                action=action,
+                error=error,
+                pre_log_results=pre_log_results,
+                changes=changes,
+                log_entry=log_entry,
+                log_created=log_entry is not None,
+            )
+        if error:
+            raise error
+
+
 def make_log_m2m_changes(field_name):
     """Return a handler for m2m_changed with field_name enclosed."""
 
@@ -110,9 +156,11 @@ def make_log_m2m_changes(field_name):
             return
 
         if action == "post_clear":
-            changed_queryset = kwargs["model"].objects.all()
+            changed_queryset = kwargs["model"]._default_manager.all()
         else:
-            changed_queryset = kwargs["model"].objects.filter(pk__in=kwargs["pk_set"])
+            changed_queryset = kwargs["model"]._default_manager.filter(
+                pk__in=kwargs["pk_set"]
+            )
 
         if action in ["post_add"]:
             LogEntry.objects.log_m2m_changes(
