@@ -1,19 +1,10 @@
 import copy
 from collections import defaultdict
-from typing import (
-    Any,
-    Callable,
-    Collection,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Tuple,
-    Union,
-)
+from collections.abc import Collection, Iterable
+from typing import Any, Callable, Optional, Union
 
 from django.apps import apps
-from django.db.models import Model
+from django.db.models import ManyToManyField, Model
 from django.db.models.base import ModelBase
 from django.db.models.signals import (
     ModelSignal,
@@ -26,7 +17,7 @@ from django.db.models.signals import (
 from auditlog.conf import settings
 from auditlog.signals import accessed
 
-DispatchUID = Tuple[int, int, int]
+DispatchUID = tuple[int, int, int]
 
 
 class AuditLogRegistrationError(Exception):
@@ -47,7 +38,7 @@ class AuditlogModelRegistry:
         delete: bool = True,
         access: bool = True,
         m2m: bool = True,
-        custom: Optional[Dict[ModelSignal, Callable]] = None,
+        custom: Optional[dict[ModelSignal, Callable]] = None,
     ):
         from auditlog.receivers import log_access, log_create, log_delete, log_update
 
@@ -71,13 +62,13 @@ class AuditlogModelRegistry:
     def register(
         self,
         model: ModelBase = None,
-        include_fields: Optional[List[str]] = None,
-        exclude_fields: Optional[List[str]] = None,
-        mapping_fields: Optional[Dict[str, str]] = None,
-        mask_fields: Optional[List[str]] = None,
+        include_fields: Optional[list[str]] = None,
+        exclude_fields: Optional[list[str]] = None,
+        mapping_fields: Optional[dict[str, str]] = None,
+        mask_fields: Optional[list[str]] = None,
         m2m_fields: Optional[Collection[str]] = None,
         serialize_data: bool = False,
-        serialize_kwargs: Optional[Dict[str, Any]] = None,
+        serialize_kwargs: Optional[dict[str, Any]] = None,
         serialize_auditlog_fields_only: bool = False,
     ):
         """
@@ -113,6 +104,12 @@ class AuditlogModelRegistry:
                 "set. Did you forget to set serialized_data to True?"
             )
 
+        for fld in settings.AUDITLOG_EXCLUDE_TRACKING_FIELDS:
+            exclude_fields.append(fld)
+
+        for fld in settings.AUDITLOG_MASK_TRACKING_FIELDS:
+            mask_fields.append(fld)
+
         def registrar(cls):
             """Register models for a given class."""
             if not issubclass(cls, Model):
@@ -166,7 +163,7 @@ class AuditlogModelRegistry:
         else:
             self._disconnect_signals(model)
 
-    def get_models(self) -> List[ModelBase]:
+    def get_models(self) -> list[ModelBase]:
         return list(self._registry.keys())
 
     def get_model_fields(self, model: ModelBase):
@@ -208,7 +205,7 @@ class AuditlogModelRegistry:
                 m2m_changed.connect(
                     receiver,
                     sender=m2m_model,
-                    dispatch_uid=self._dispatch_uid(m2m_changed, receiver),
+                    dispatch_uid=self._m2m_dispatch_uid(m2m_changed, m2m_model),
                 )
 
     def _disconnect_signals(self, model):
@@ -224,7 +221,7 @@ class AuditlogModelRegistry:
             m2m_model = getattr(field, "through")
             m2m_changed.disconnect(
                 sender=m2m_model,
-                dispatch_uid=self._dispatch_uid(m2m_changed, receiver),
+                dispatch_uid=self._m2m_dispatch_uid(m2m_changed, m2m_model),
             )
         del self._m2m_signals[model]
 
@@ -232,7 +229,11 @@ class AuditlogModelRegistry:
         """Generate a dispatch_uid which is unique for a combination of self, signal, and receiver."""
         return id(self), id(signal), id(receiver)
 
-    def _get_model_classes(self, app_model: str) -> List[ModelBase]:
+    def _m2m_dispatch_uid(self, signal, sender) -> DispatchUID:
+        """Generate a dispatch_uid which is unique for a combination of self, signal, and sender."""
+        return id(self), id(signal), id(sender)
+
+    def _get_model_classes(self, app_model: str) -> list[ModelBase]:
         try:
             try:
                 app_label, model_name = app_model.split(".")
@@ -244,15 +245,16 @@ class AuditlogModelRegistry:
 
     def _get_exclude_models(
         self, exclude_tracking_models: Iterable[str]
-    ) -> List[ModelBase]:
+    ) -> list[ModelBase]:
         exclude_models = [
             model
-            for app_model in exclude_tracking_models + self.DEFAULT_EXCLUDE_MODELS
+            for app_model in tuple(exclude_tracking_models)
+            + self.DEFAULT_EXCLUDE_MODELS
             for model in self._get_model_classes(app_model)
         ]
         return exclude_models
 
-    def _register_models(self, models: Iterable[Union[str, Dict[str, Any]]]) -> None:
+    def _register_models(self, models: Iterable[Union[str, dict[str, Any]]]) -> None:
         models = copy.deepcopy(models)
         for model in models:
             if isinstance(model, str):
@@ -260,7 +262,13 @@ class AuditlogModelRegistry:
                     self.unregister(model_class)
                     self.register(model_class)
             elif isinstance(model, dict):
-                model["model"] = self._get_model_classes(model["model"])[0]
+                appmodel = self._get_model_classes(model["model"])
+                if not appmodel:
+                    raise AuditLogRegistrationError(
+                        f"An error was encountered while registering model '{model['model']}' - "
+                        "make sure the app is registered correctly."
+                    )
+                model["model"] = appmodel[0]
                 self.unregister(model["model"])
                 self.register(**model)
 
@@ -286,11 +294,39 @@ class AuditlogModelRegistry:
                 "setting 'AUDITLOG_INCLUDE_ALL_MODELS' must set to 'True'"
             )
 
+        if (
+            settings.AUDITLOG_EXCLUDE_TRACKING_FIELDS
+            and not settings.AUDITLOG_INCLUDE_ALL_MODELS
+        ):
+            raise ValueError(
+                "In order to use 'AUDITLOG_EXCLUDE_TRACKING_FIELDS', "
+                "setting 'AUDITLOG_INCLUDE_ALL_MODELS' must be set to 'True'"
+            )
+
+        if (
+            settings.AUDITLOG_MASK_TRACKING_FIELDS
+            and not settings.AUDITLOG_INCLUDE_ALL_MODELS
+        ):
+            raise ValueError(
+                "In order to use 'AUDITLOG_MASK_TRACKING_FIELDS', "
+                "setting 'AUDITLOG_INCLUDE_ALL_MODELS' must be set to 'True'"
+            )
+
         if not isinstance(settings.AUDITLOG_INCLUDE_TRACKING_MODELS, (list, tuple)):
             raise TypeError(
                 "Setting 'AUDITLOG_INCLUDE_TRACKING_MODELS' must be a list or tuple"
             )
 
+        if not isinstance(settings.AUDITLOG_EXCLUDE_TRACKING_FIELDS, (list, tuple)):
+            raise TypeError(
+                "Setting 'AUDITLOG_EXCLUDE_TRACKING_FIELDS' must be a list or tuple"
+            )
+
+        if not isinstance(settings.AUDITLOG_MASK_TRACKING_FIELDS, (list, tuple)):
+            raise TypeError(
+                "Setting 'AUDITLOG_MASK_TRACKING_FIELDS' must be a list or tuple"
+            )
+
         for item in settings.AUDITLOG_INCLUDE_TRACKING_MODELS:
             if not isinstance(item, (str, dict)):
                 raise TypeError(
@@ -312,12 +348,28 @@ class AuditlogModelRegistry:
             exclude_models = self._get_exclude_models(
                 settings.AUDITLOG_EXCLUDE_TRACKING_MODELS
             )
-            models = apps.get_models()
 
-            for model in models:
+            for model in apps.get_models(include_auto_created=True):
                 if model in exclude_models:
                     continue
-                self.register(model)
+
+                meta = model._meta
+                if not meta.managed:
+                    continue
+
+                m2m_fields = [
+                    m.name for m in meta.get_fields() if isinstance(m, ManyToManyField)
+                ]
+
+                exclude_fields = [
+                    i.related_name
+                    for i in meta.related_objects
+                    if i.related_name and not i.related_model._meta.managed
+                ]
+
+                self.register(
+                    model=model, m2m_fields=m2m_fields, exclude_fields=exclude_fields
+                )
 
         self._register_models(settings.AUDITLOG_INCLUDE_TRACKING_MODELS)
 
